package com.gic.finance.service.outer.impl;

import java.util.Date;

import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import com.gic.finance.constant.InvoiceInitiateTypeConstants;
import com.gic.finance.constant.InvoiceStatusEnum;
import com.gic.finance.dto.InvoiceManageDTO;
import com.gic.finance.dto.callback.InvoiceCallbackDTO;
import com.gic.finance.entity.TabInvoiceAccount;
import com.gic.finance.entity.TabInvoiceManage;
import com.gic.finance.qo.InvoiceManageListQueryQO;
import com.gic.finance.service.InvoiceAccountService;
import com.gic.finance.service.InvoiceManageApiService;
import com.gic.finance.service.InvoiceManageService;
import com.gic.finance.util.AsynCallbackUtils;
import com.gic.store.utils.CreateSerialNumberUtils;

@Service("invoiceManageApiService")
public class InvoiceManageApiServiceImpl implements InvoiceManageApiService {
    private static final Logger LOGGER = LogManager.getLogger(InvoiceManageApiServiceImpl.class);
    @Autowired
    private InvoiceManageService invoiceManageService;
    @Autowired
    private InvoiceAccountService invoiceAccountService;

    public ServiceResponse<Void> applyInvoiceInit(InvoiceManageDTO invoiceManageDTO) {
        ServiceResponse paramResponse = ValidParamsUtils.allCheckValidate(invoiceManageDTO,
                InvoiceManageDTO.ApplyInvoiceValid.class);
        if (!paramResponse.isSuccess()) {
            return paramResponse;
        }
        if (invoiceManageDTO.getBillingAmount() < 1000) {
            //如果开票金额小于1000，则不允许开票
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "开票金额小于1000");
        }
        invoiceManageDTO.setCreateTime(new Date());
        invoiceManageDTO.setUpdateTime(new Date());
        invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.TO_BE_ISSUED.getCode());
        invoiceManageDTO.setInvoiceApplySerial(CreateSerialNumberUtils.createSerialNumber());
        if (StringUtils.isBlank(invoiceManageDTO.getEnterpriseName())) {
            invoiceManageDTO.setEnterpriseName("暂无商户名称");
        }
        if (StringUtils.isBlank(invoiceManageDTO.getCompanyName())) {
            invoiceManageDTO.setCompanyName("暂无公司名称");
        }
        if (StringUtils.isBlank(invoiceManageDTO.getLogo())) {
            invoiceManageDTO.setLogo("暂无公司logo");
        }

        invoiceManageService.save(invoiceManageDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> applyInvoice(InvoiceManageDTO invoiceManageDTO) {
        //计费中心
        invoiceManageDTO.setInitiateType(InvoiceInitiateTypeConstants.BILLING);
        return applyInvoiceInit(invoiceManageDTO);
    }

    @Override
    public ServiceResponse<Void> applyInvoiceForApp(InvoiceManageDTO invoiceManageDTO) {
        if (StringUtils.isBlank(invoiceManageDTO.getInitiator())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发起人(initiator)主键不能为空");
        }
        //应用市场类型
        invoiceManageDTO.setInitiateType(InvoiceInitiateTypeConstants.APP);
        return applyInvoiceInit(invoiceManageDTO);
    }

    @Override
    public ServiceResponse<Void> invoice(Integer invoiceManageId, String expressMailName, String expressMailNumber) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        InvoiceManageDTO invoiceManageDTO = new InvoiceManageDTO();
        invoiceManageDTO.setInvoiceManageId(invoiceManageId);
        if (StringUtils.isBlank(expressMailName) || StringUtils.isBlank(expressMailNumber)) {
            //待邮寄
            invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.TO_BE_MAILED.getCode());
            tabInvoiceManage.setInvoiceStatus(InvoiceStatusEnum.TO_BE_MAILED.getCode());
        } else {
            invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.MAILED.getCode());
            //物流信息
            invoiceManageDTO.setExpressMailName(expressMailName);
            invoiceManageDTO.setExpressMailNumber(expressMailNumber);

            tabInvoiceManage.setInvoiceStatus(InvoiceStatusEnum.MAILED.getCode());
            tabInvoiceManage.setExpressMailName(expressMailName);
            tabInvoiceManage.setExpressMailNumber(expressMailNumber);
        }
        //todo 操作人相关信息

        //开票户信息插入
        TabInvoiceAccount account = invoiceAccountService.getEnable();
        if (account == null) {
            return ServiceResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "没有启用的开票户信息，请启用");
        }
        invoiceManageDTO.setInvoicerId(account.getInvoiceAccountId());
        invoiceManageService.update(invoiceManageDTO);

        //回调
        if (tabInvoiceManage.getInitiateType().intValue() == InvoiceInitiateTypeConstants.APP) {
            InvoiceCallbackDTO callbackDTO = EntityUtil.changeEntityNew(InvoiceCallbackDTO.class, tabInvoiceManage);
            AsynCallbackUtils.callBack(callbackDTO, "com.gic.open.api.service.market.InvoiceCallbackApiService",
                    "invoiceCallback");
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> rejectInvoice(Integer invoiceManageId, String rejectReason) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        InvoiceManageDTO invoiceManageDTO = new InvoiceManageDTO();
        invoiceManageDTO.setInvoiceManageId(invoiceManageId);
        invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.REJECT.getCode());
        if (StringUtils.isBlank(rejectReason)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "驳回理由不能为空");
        }
        invoiceManageDTO.setRejectReason(rejectReason);
        //todo 操作人相关信息

        invoiceManageService.update(invoiceManageDTO);

        //回调
        if (tabInvoiceManage.getInitiateType().intValue() == InvoiceInitiateTypeConstants.APP) {
            tabInvoiceManage.setInvoiceStatus(InvoiceStatusEnum.REJECT.getCode());
            tabInvoiceManage.setRejectReason(rejectReason);
            InvoiceCallbackDTO callbackDTO = EntityUtil.changeEntityNew(InvoiceCallbackDTO.class, tabInvoiceManage);
            AsynCallbackUtils.callBack(callbackDTO, "com.gic.open.api.service.market.InvoiceCallbackApiService",
                    "invoiceCallback");
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> cancelInvoice(Integer invoiceManageId) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        InvoiceManageDTO invoiceManageDTO = new InvoiceManageDTO();
        invoiceManageDTO.setInvoiceManageId(invoiceManageId);
        invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.CANCEL.getCode());
        invoiceManageService.update(invoiceManageDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<InvoiceManageDTO>> listInvoiceManage(InvoiceManageListQueryQO params) {
        com.github.pagehelper.Page page = invoiceManageService.listInvoiceManage(params);
        Page<InvoiceManageDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page, InvoiceManageDTO.class);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<InvoiceManageDTO> getById(Integer invoiceManageId) {
        TabInvoiceManage tabInvoiceManage = invoiceManageService.getById(invoiceManageId);
        if (tabInvoiceManage == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发票管理主键有误");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(InvoiceManageDTO.class, tabInvoiceManage));
    }

    @Override
    public ServiceResponse<Void> cancelInvoiceForApp(String initiator) {
        LOGGER.info("应用市场取消开票：{}", initiator);
        TabInvoiceManage record = invoiceManageService.selectInvoiceByInitiator(initiator,
                InvoiceInitiateTypeConstants.APP);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数有误，查无数据");
        }
        if (record.getInvoiceStatus().intValue() != InvoiceStatusEnum.TO_BE_ISSUED.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(),
                    "不是" + InvoiceStatusEnum.TO_BE_ISSUED.getMessage() + "状态，无法取消");
        }
        InvoiceManageDTO invoiceManageDTO = new InvoiceManageDTO();
        invoiceManageDTO.setInvoiceManageId(record.getInvoiceManageId());
        invoiceManageDTO.setInvoiceStatus(InvoiceStatusEnum.CANCEL.getCode());
        invoiceManageService.update(invoiceManageDTO);
        return ServiceResponse.success();
    }
}
